// jack_test.cpp
//

#include <jack/jack.h>
#include <iostream>
#include <sstream>
#include <unistd.h>

const int NPORTS = 4;

jack_client_t *client;
jack_port_t *iport[NPORTS];
jack_port_t *oport[NPORTS];

int process(jack_nframes_t frames, void *arg) {
	// std::cout << "process callback" << std::endl;
	jack_default_audio_sample_t *ibuf;
	jack_default_audio_sample_t *obuf;
	for (int i = 0; i < NPORTS; i++) {
		obuf = (jack_default_audio_sample_t*) jack_port_get_buffer(oport[i], frames);
		for (int j = 0; j < NPORTS; j++) {
			ibuf = (jack_default_audio_sample_t*) jack_port_get_buffer(iport[j], frames);
			for (jack_nframes_t frame = 0; frame < frames; frame++) {
				if (j == 0) obuf[frame] = 0.0; 
				obuf[frame] += ibuf[frame] / (float) NPORTS;
			}
		}
	}
	return 1;
}

int main(int argc, char *argv[]) {
	// default = 60 seconds
	unsigned int seconds_to_run = 60;
	if (argc > 1) {
		std::stringstream sec_stream;
		sec_stream << argv[1];
		sec_stream >> seconds_to_run;
	}
	std::cout << "seconds to run: " << seconds_to_run << std::endl;
	std::stringstream pid_stream;
	pid_stream << "jack_test-" << getpid();

	std::cout << "client_new: " << pid_stream.str() << std::endl;
	client = jack_client_new(pid_stream.str().c_str());
	if (!client) {
		std::cerr << "jackd not running?" << std::endl;
		return 1;
	}

	std::cout << "port_register" << std::endl;
	for (int i = 0; i < NPORTS; i++) {
		std::stringstream iport_name;
		iport_name << "in_" << i;
		iport[i] = jack_port_register(client,  iport_name.str().c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput|JackPortIsTerminal, 0);
		std::stringstream oport_name;
		oport_name << "out_" << i;
		oport[i] = jack_port_register(client, oport_name.str().c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsTerminal|JackPortIsOutput, 0);
	}
	std::cout << "set_process_callback" << std::endl;
	jack_set_process_callback(client, process, 0);

	std::cout << "activate" << std::endl;
	jack_activate(client);

	std::cout << "running" << std::endl;

	// while(1) {sleep(1);};
	sleep(seconds_to_run);

	jack_deactivate(client);
	jack_client_close(client);
	
	return 0;
}

// end of jack_test.cpp
